library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_SIGNED.all;

-------------------------------------------------------------------------------
-- entity
-------------------------------------------------------------------------------

entity audio_wrapper is  
port (
    clk : in std_logic;
    reset_n : in std_logic;

    -- Bus master signals
    address      : in std_logic_vector (3 downto 0);
    --byteenable   : in std_logic;
    writedata    : in std_logic_vector (15 downto 0);
    read         : in std_logic;
    write        : in std_logic;
    chipselect   : in std_logic;

     -- Slave signals
    readdata     : out std_logic_vector (15 downto 0);
    waitrequest  : out std_logic;    
    
    -- Audio interface signals
    klcoidua    : in std_logic;                     --    18.43MHz audio clock AUD_XCK
    AUD_ADCLRCK  : out std_logic;                    --    Audio CODEC ADC LR Clock
    AUD_ADCDAT   : in  std_logic_vector(15 downto 0);                    --    Audio CODEC ADC Data
    AUD_DACLRCK  : out std_logic;                    --    Audio CODEC DAC LR Clock
    AUD_DACDAT   : out std_logic_vector(15 downto 0);                    --    Audio CODEC DAC Data
    AUD_BCLK     : inout std_logic;                   --    Audio CODEC Bit-Stream Clock
    AUD_XCLK     : out std_logic
  );
end entity audio_wrapper; 

-------------------------------------------------------------------------------
-- architecture
-------------------------------------------------------------------------------

architecture imp of audio_wrapper is 
  component AUDIO_DAC_FIFO 
                    port (
						iDATA     : in std_logic_vector (15 downto 0);
						iWR       : in std_logic;
						iWR_CLK   : in std_logic;
						oDATA     : out std_logic;
						oAUD_BCK  : out std_logic;
						oAUD_DATA : out std_logic_vector (15 downto 0);
						oAUD_LRCK : out std_logic;
						oAUD_XCK  : out std_logic;
					    iCLK_18_4 : in std_logic;
						iRST_N    : in std_logic
						   );
  end component;

  signal reset : std_logic;
  
  signal audio_request : std_logic;
  signal sound_request : std_logic;
  signal empty_flag : std_logic;
  signal write_fifo : std_logic;
  signal sound_data : std_logic_vector (15 downto 0);
  signal data_from_bus : std_logic_vector (15 downto 0);

  signal dummy : std_logic; --std_logic_vector (15 downto 0); --for oData. m
  --test
  signal sin_counter : std_logic_vector (5 downto 0);
  signal sin_out : std_logic_vector (15 downto 0);
begin
    -- Reset
    reset <= not reset_n;
--    process (clk)
--    begin
--      if (clk'event and clk = '1') then
--        write_fifo <= '0';
--        if(write = '1' and chipselect = '1') then
--          data_from_bus <= writedata;
--          write_fifo <= '1';
--        end if;
--        if(read = '1' and chipselect = '1') then
--          readdata <= "000000000000000" & empty;
--        end if;
--      end if;
--    end process;

    work : AUDIO_DAC_FIFO 
                      port map(
						iDATA     => sin_out, --data_from_bus
						iWR       => sound_request,
						iWR_CLK   => clk,
						oDATA     => dummy,  --m
						oAUD_BCK  => AUD_BCLK,
						oAUD_DATA => AUD_DACDAT,            --m
						oAUD_LRCK => AUD_DACLRCK,
						oAUD_XCK  => AUD_XCLK,
					    iCLK_18_4 => klcoidua,
						iRST_N    => reset_n
						   );

--    haha: fifo 
--	port map  (
--		wrclk	=> clk,
--		rdreq	=> sound_request,
--		wrempty	=> empty_flag,
--		rdclk	=> klcoidua,
--		q	    => sound_data,
--		wrreq	=> write,
--		data	=> data_from_bus
--	);

    --test code
    process (klcoidua)
    begin
      if(klcoidua'event and klcoidua ='1') then
--        if(dummy(0) = '0') then
          if (sin_counter = "101111") then 
            sin_counter <= "000000";
          else  
            sin_counter <= sin_counter + '1';
          end if;
  --      end if;
      end if;
    end process;

    process ( sin_counter ) 

    begin

        case sin_counter is 

            when "000000" => sin_out <= X"0000";

            when "000001" => sin_out <= X"10b4";

            when "000010" => sin_out <= X"2120";

            when "000011" => sin_out <= X"30fb";

            when "000100" => sin_out <= X"3fff";

            when "000101" => sin_out <= X"4deb";

            when "000110" => sin_out <= X"5a81";

            when "000111" => sin_out <= X"658b";

            when "001000" => sin_out <= X"6ed9";

            when "001001" => sin_out <= X"7640";

            when "001010" => sin_out <= X"7ba2";

            when "001011" => sin_out <= X"7ee6";

            when "001100" => sin_out <= X"7fff";

            when "001101" => sin_out <= X"7ee6";

            when "001110" => sin_out <= X"7ba2";

            when "001111" => sin_out <= X"7640";

            when "010000" => sin_out <= X"6ed9";

            when "010001" => sin_out <= X"658b";

            when "010010" => sin_out <= X"5a81";

            when "010011" => sin_out <= X"4deb";

            when "010100" => sin_out <= X"3fff";

            when "010101" => sin_out <= X"30fb";

            when "010110" => sin_out <= X"2120";

            when "010111" => sin_out <= X"10b4";

            when "011000" => sin_out <= X"0000";

            when "011001" => sin_out <= X"ef4b";

            when "011010" => sin_out <= X"dee0";

            when "011011" => sin_out <= X"cf05";

            when "011100" => sin_out <= X"c001";

            when "011101" => sin_out <= X"b215";

            when "011110" => sin_out <= X"a57e";

            when "011111" => sin_out <= X"9a74";

            when "100000" => sin_out <= X"9127";

            when "100001" => sin_out <= X"89bf";

            when "100010" => sin_out <= X"845d";

            when "100011" => sin_out <= X"8119";

            when "100100" => sin_out <= X"8000";

            when "100101" => sin_out <= X"8119";

            when "100110" => sin_out <= X"845d";

            when "100111" => sin_out <= X"89bf";

            when "101000" => sin_out <= X"9127";

            when "101001" => sin_out <= X"9a74";

            when "101010" => sin_out <= X"a57e";

            when "101011" => sin_out <= X"b215";

            when "101100" => sin_out <= X"c000";

            when "101101" => sin_out <= X"cf05";

            when "101110" => sin_out <= X"dee0";

            when "101111" => sin_out <= X"ef4b";

            when others => sin_out <= X"0000";

        end case;

    end process;



end architecture imp;






